﻿local appname = "Broker_repair"
local app = LibStub("AceAddon-3.0"):NewAddon(appname, "AceEvent-3.0", "AceBucket-3.0")
app.version = GetAddOnMetadata(appname, "Version") or '0.0'
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale("Broker_repair",true)
local Crayon = AceLibrary("Crayon-2.0")
local Abacus = AceLibrary("Abacus-2.0")
local Gratuity = AceLibrary("Gratuity-2.0")
local dataobj
local OnEvent

local ldb = LibStub:GetLibrary("LibDataBroker-1.1",true)
dataobj = ldb:NewDataObject("Broker_repair", {
	type  = "data source",
	label = L["Broker Repair"],
	icon  = "Interface\\Addons\\Broker_repair\\media\\icon",
	text  = "",
})

local Broker_repair_Frame = CreateFrame("Frame")
local Broker_repair_Option = CreateFrame("Frame","Broker_repair_Option",UIParent);

local autoRepairState
local guildRepairState
local junkAutoSellState
local costShowState
local costTypeState

local slotNames = {
	{ 0, 0, 0, "Head", L["Head"] },
	{ 0, 0, 0, "Neck", L["Neck"] },
	{ 0, 0, 0, "Shoulder", L["Shoulder"] },
	{ 0, 0, 0, "Back", L["Back"] },
	{ 0, 0, 0, "Chest", L["Chest"] },
	{ 0, 0, 0, "Shirt", L["Shirt"] },
	{ 0, 0, 0, "Tabard", L["Tabard"] },
	{ 0, 0, 0, "Wrist", L["Wrist"] },
	{ 0, 0, 0, "Hands", L["Hands"] },
	{ 0, 0, 0, "Waist", L["Waist"] },
	{ 0, 0, 0, "Legs", L["Legs"] },
	{ 0, 0, 0, "Feet", L["Feet"] },
	{ 0, 0, 0, "Finger0", L["Finger0"] },
	{ 0, 0, 0, "Finger1", L["Finger1"] },
	{ 0, 0, 0, "Trinket0", L["Trinket0"] },
	{ 0, 0, 0, "Trinket1", L["Trinket1"] },
	{ 0, 0, 0, "MainHand", L["MainHand"] },
	{ 0, 0, 0, "SecondaryHand", L["SecondaryHand"] },
	{ 0, 0, 0, "Ranged", L["Ranged"] },
	{ 0, 0, 0, "Bag0", L["Bag0"] },
	{ 0, 0, 0, "Bag1", L["Bag1"] },
	{ 0, 0, 0, "Bag2", L["Bag2"] },
	{ 0, 0, 0, "Bag3", L["Bag3"] },
}

local colorWhite = "|cFFFFFFFF";
local colorGray  = "|cFFAAAAAA";

local autoRepairLine  = nil
local guildRepairLine = nil
local junkAutoSellLine= nil
local costShowLine    = nil
local costTypeLine    = nil
local factionLine     = { }

local VALUE = 1
local MAX   = 2
local COST  = 3
local SLOT  = 4
local NAME  = 5
local total = 0
local cur   = 0

local states = {
	autoRepair = {
		default = 0,
		[0] = {
			color     = "|cFFFF0000",
			status    = L["OFF"],
			nextState = 1,
		},
		[1] = {
			color     = "|cFF00FF00",
			status    = L["ON"],
			nextState = 0,
		},
	},
	guildRepair = {
		default = 0,
		[0] = {
			color     = "|cFFFF0000",
			status    = L["OFF"],
			nextState = 1,
		},
		[1] = {
			color     = "|cFF00FF00",
			status    = L["ON"],
			nextState = 0,
		},
	},
	junkAutoSell = {
		default = 0,
		[0] = {
			color     = "|cFFFF0000",
			status    = L["OFF"],
			nextState = 1,
		},
		[1] = {
			color     = "|cFF00FF00",
			status    = L["ON"],
			nextState = 0,
		},
	},
	costShow = {
		default = 0,
		[0] = {
			color     = "|cFFAAAAAA",
			status    = L["ONLY TOOLTIP"],
			nextState = 1,
		},
		[1] = {
			color     = "|cFFAAAAAA",
			status    = L["PLUGIN / TOOLTIP"],
			nextState = 0,
		},
	},
	costType = {
		default = 0,
		[0] = {
			color     = "|cFFAAAAAA",
			status    = L["TEXT"],
			nextState = 1,
		},
		[1] = {
			color     = "|cFFAAAAAA",
			status    = L["ICON"],
			nextState = 0,
		},
	},
}

local CopperToString = function(c,type)
	if c == 0 then return "" end
	local str = ""
	if not c or c < 0 then return str end
	if not Broker_repairDB.costType or Broker_repairDB.costType == 0 then
		str = Abacus:FormatMoneyFull(c, true)
	else
		local coinHeight = 0
		if type == "tip" then
			coinHeight = 0
		elseif type == "msg" then
			coinHeight = -3
		end
	
		if c >= 10000 then
			local g = math.floor(c/10000)
			c = c - g*10000
			str = str.."|cFFFFD800"..g.." |TInterface\\MoneyFrame\\UI-GoldIcon.blp:0:0:0:"..coinHeight.."|t"
		end
		if c >= 100 then
			local s = math.floor(c/100)
			c = c - s*100
			str = str.."|cFFC7C7C7"..s.." |TInterface\\MoneyFrame\\UI-SilverIcon.blp:0:0:0:"..coinHeight.."|t"
		end
		if c >= 0 then
			str = str.."|cFFEEA55F"..c.." |TInterface\\MoneyFrame\\UI-CopperIcon.blp:0:0:0:"..coinHeight.."|t"
		end
	end
	return str
end

function dataobj.OnTooltipShow(tip)
	if not tip or not tip.AddLine or not tip.AddDoubleLine then return end
	if not myTip then myTip = CreateFrame("GameTooltip", "Broker_repair_Tip") end
	
	local totalcost = 0
	tip:AddLine(colorWhite.."Broker repair v"..app.version.." LovelyEdition")
	tip:AddLine(colorGray.."Edit by KIMBRA RAVENHOLDT (EU)")
	tip:AddLine(" ")

	autoRepairState   = app:GetState("autoRepair")
	guildRepairState  = app:GetState("guildRepair")
	junkAutoSellState = app:GetState("junkAutoSell")
	costShowState     = app:GetState("costShow")
	costTypeState     = app:GetState("costType")
	tip:AddLine(colorWhite..L["Options"])
	autoRepairLine  = tip:AddDoubleLine("|cFFFFCC00"..L["Auto Repair"]..":",autoRepairState.color..autoRepairState.status)
	guildRepairLine = tip:AddDoubleLine("|cFFFFCC00"..L["Guild Repair"]..":",guildRepairState.color..guildRepairState.status)
	costTypeLine    = tip:AddDoubleLine("|cFFFFCC00"..L["Cost Icon"]..":",costTypeState.color..costTypeState.status)
	junkAutoSellLine= tip:AddDoubleLine("|cFFFFCC00"..L["junk Auto sell"]..":",junkAutoSellState.color..junkAutoSellState.status)
	costShowLine    = tip:AddDoubleLine("|cFFFFCC00"..L["Cost Show"]..":",costShowState.color..costShowState.status)
	tip:AddLine(" ")

	local totalcost,_,_  = app:GetRepairData()
	if totalcost <= 0 then
		tip:AddLine(L["All item good!"],0,1,0)
	else
		tip:AddLine(colorWhite..L["Status"])
		for index,item in pairs(slotNames) do
			if item[MAX] > 0 and item[VALUE] < item[MAX] then
				local percent = app:GetPercent(item[VALUE], item[MAX])
				local r, g, b = Crayon:GetThresholdColor(percent)
				tip:AddDoubleLine(colorGray..item[NAME], string.format("%.0f%%", percent * 100), 1, 1, 1, r, g, b)
			end
		end
		local percent = app:GetPercent(cur, total)
		local r, g, b = Crayon:GetThresholdColor(percent)
		tip:AddLine(" ")
		tip:AddDoubleLine(colorGray..L["Total"].." :", string.format("%.0f%%", percent * 100), 1, 1, 1, r, g, b)

		tip:AddLine(" ")
		tip:AddLine(colorWhite..L["Cost for faction"])
		local m = 1
	
		for i=4, 8 do
			tip:AddDoubleLine(colorGray.._G["FACTION_STANDING_LABEL"..i], CopperToString(math.floor(totalcost*m+.5), "tip"))
			m = m - .05
		end
	end

	tip:AddLine(" ")
	tip:AddLine(colorWhite..L["Available Mouse Shortcuts..."])
	tip:AddLine("|cFFFFCC00"..L["LeftClick"]..": |c00FFFFFF"..L["Set Auto Repair"])
	tip:AddLine("|cFFFFCC00"..L["SHIFT"].." + "..L["LeftClick"]..": |c00FFFFFF"..L["Set Guild Repair"])
	tip:AddLine("|cFFFFCC00"..L["ALT"].." + "..L["LeftClick"]..": |c00FFFFFF"..L["Set Cost Icon"])
	tip:AddLine("|cFFFFCC00"..L["RightClick"]..": |c00FFFFFF"..L["Set Junk Auto Sell"])
	tip:AddLine("|cFFFFCC00"..L["CTRL"].." + "..L["LeftClick"]..": |c00FFFFFF"..L["Set Cost Show"])
	
end

-- events registration
function app:OnInitialize()
	if not Broker_repairDB then
		Broker_repairDB = { }
	end
	for key,state in pairs(states) do
		if not Broker_repairDB[key] or not states[key][ Broker_repairDB[key] ] then
			Broker_repairDB[key] = state.default
		end
	end

	app:RegisterEvent("MERCHANT_SHOW")
	app:RegisterBucketEvent({"UPDATE_INVENTORY_DURABILITY","UPDATE_INVENTORY_ALERTS","UNIT_INVENTORY_CHANGED","EQUIPMENT_SWAP_FINISHED","PLAYER_DEAD","PLAYER_UNGHOST","MERCHANT_CLOSED","MERCHANT_SHOW","PLAYER_REGEN_ENABLED"},2,"OnDataUpdate")
	app:OnDataUpdate()
end

-- Local function
function app:GetState(key)
	assert(states[key], "Unknown state: "..(key or "nil"))
	local currentState = Broker_repairDB[key] or states[key].default
	return states[key][currentState]
end

function app:SetNextState(key)
	local currentState = app:GetState(key)
	Broker_repairDB[key] = currentState.nextState or 0
	return app:GetState(key)
end

function dataobj.OnClick(self, button)
	if button == "LeftButton" then
		if IsAltKeyDown() then
			local state = app:SetNextState("costType")
			DEFAULT_CHAT_FRAME:AddMessage("|CFFffff00"..L["Cost Icon"].."|r: "..state.color..state.status)
		elseif IsShiftKeyDown() then
			local state = app:SetNextState("guildRepair")
			DEFAULT_CHAT_FRAME:AddMessage("|CFFffff00"..L["Guild Repair"].."|r: "..state.color..state.status)
		elseif IsControlKeyDown() then
			local state = app:SetNextState("costShow")
			DEFAULT_CHAT_FRAME:AddMessage("|CFFffff00"..L["Cost Show"].."|r: "..state.color..state.status)
		else
			local state = app:SetNextState("autoRepair")
			DEFAULT_CHAT_FRAME:AddMessage("|CFFffff00"..L["Auto Repair"].."|r: "..state.color..state.status)
		end

	elseif button == "RightButton" then
		local state = app:SetNextState("junkAutoSell")
		DEFAULT_CHAT_FRAME:AddMessage("|CFFffff00"..L["junk Auto sell"].."|r: "..state.color..state.status)
	end
	app:OnDataUpdate()
end

local JunkAutoSell = function()
	if not Broker_repairDB.junkAutoSell or Broker_repairDB.junkAutoSell == 0 then return end
	local sellingCount = 0
	local sellMoney = 0
	for bag=0,4 do
		if(GetContainerNumSlots(bag)) then
			for slot=1,GetContainerNumSlots(bag) do
				local _,count,_,qual = GetContainerItemInfo(bag,slot)
				if(qual and ((qual == 0) or (string.match(GetContainerItemLink(bag,slot), ITEM_QUALITY_COLORS[0].hex)))) then
					local item = GetContainerItemLink(bag,slot)
					local sellPrice = select(11,GetItemInfo(item))
					if(count > 1) then
						DEFAULT_CHAT_FRAME:AddMessage("|CFF008406"..L["You sell to vendor "]..":|r "..item.."|CFF008406 x"..count.."|r")
						sellMoney = sellMoney + (sellPrice*count)
					else
						DEFAULT_CHAT_FRAME:AddMessage("|CFF008406"..L["You sell to vendor "]..":|r "..item)
						sellMoney = sellMoney + sellPrice
					end
					UseContainerItem(bag,slot)
					sellingCount = sellingCount +1
				end
			end
		end
	end
	if(sellingCount == 0) then
		DEFAULT_CHAT_FRAME:AddMessage("|CFFffff00"..L["You do not sell to vendor."].."|r")
	else
		DEFAULT_CHAT_FRAME:AddMessage("|CFFffff00"..L["You sell for "]..": |r"..CopperToString(sellMoney,"msg"))
	end
end

local AutoRepair = function()
	if not Broker_repairDB.autoRepair or Broker_repairDB.autoRepair == 0 or not CanMerchantRepair() then return end
	local cost, canRepair = GetRepairAllCost()
	if not canRepair or cost == 0 then
		DEFAULT_CHAT_FRAME:AddMessage("|CFFffff00"..L["You do not repair!"].."|r")
		return
	end
	
	if Broker_repairDB.autoRepair == 1 then
		if CanGuildBankRepair() and Broker_repairDB.guildRepair==1 then
			app:RepairWithGuildBank()
		else
			app:Repair()
		end
	end
end

function app:Repair()
	local cost = GetRepairAllCost()
	if GetMoney() >= cost then
		RepairAllItems()
		DEFAULT_CHAT_FRAME:AddMessage("|CFFffff00"..L["You repairing for"].." |r"..CopperToString(cost,"msg"))
	else
		DEFAULT_CHAT_FRAME:AddMessage("|CFFffff00"..L["You have not"].." |r"..CopperToString(cost,"msg").."|CFFffff00"..L["for repairing."].."|r")
	end
end

function app:RepairWithGuildBank()
	local cost = GetRepairAllCost()
	if GetGuildBankMoney() >= cost then
		RepairAllItems(1)
		DEFAULT_CHAT_FRAME:AddMessage("|CFFffff00"..L["You repairing for"].." |r"..CopperToString(cost,"msg").."|CFFffff00("..L["GuildBank Money"]..")|r")
	else
		DEFAULT_CHAT_FRAME:AddMessage("|CFFffff00"..L["The Guild Bank have not"].." |r"..CopperToString(cost,"msg").."|CFFffff00"..L["for your repairing."].."|r")
	end
end


function app:MERCHANT_SHOW()
	app:RepairCursorVisible(Broker_repairDB.autoRepair)
	JunkAutoSell()
	AutoRepair()
end

function app:OnDataUpdate()

	local totalcost, percent, percentmin  = app:GetRepairData()
	local r, g, b = Crayon:GetThresholdColor(percentmin)
	
	if not Broker_repairDB.costShow or Broker_repairDB.costShow == 0 then
		dataobj.text = (string.format("|cff%s%d%%|r", Crayon:GetThresholdHexColor(percentmin), percentmin * 100))
	else
		dataobj.text = (string.format("|cff%s%d%%|r", Crayon:GetThresholdHexColor(percentmin), percentmin * 100))
		.." "..CopperToString(totalcost, "tip")
	end
	
end

function app:GetRepairData()
	local totalcost = 0
	local percent = 0
	local percentmin = 1
	
	total = 0
	cur = 0
	
	for index,item in pairs(slotNames) do
		local id = GetInventorySlotInfo(item[SLOT].."Slot")
		local val, max = GetInventoryItemDurability(id)
		local hasItem, hasCooldown, repairCost = Gratuity:SetInventoryItem("player", id)
		if(max) then
			total = total + max
			cur = cur + val
			totalcost = totalcost + repairCost
			slotNames [index][VALUE] = val
			slotNames [index][MAX] = max
			slotNames [index][COST] = repairCost
			percent = app:GetPercent(val,max)
			if percent < percentmin then percentmin = percent end
		end
	end
	percent = app:GetPercent(cur,total)
	return totalcost, percent, percentmin
end

function app:GetPercent(quotient, denominator)
	if denominator ~= 0 then
		return quotient / denominator
	else
		return 1
	end
end

function app:RepairCursorVisible(repairOption)
	if repairOption == 0 then
		ShowRepairCursor()
	else
		HideRepairCursor()
	end
end